@@ -5,7 +5,6 @@
 (require 'noflet)
 (require 'el-mock)
 (require 'popup)
-
 ;;; Code:
 
 (defun dumb-jump-output-rule-test-failures (failures)
@@ -36,7 +35,7 @@
 (ert-deftest dumb-jump-get-lang-major-mode-test ()
   (let* ((major-mode 'php)
          (lang1 (dumb-jump-get-language "blah/file.install"))
-         (lang1b (dumb-jump-get-language-from-mode))
+         (lang1b (dumb-jump-get-mode-base-name))
          (lang2 (dumb-jump-get-language-by-filename "/askdfjkl/somefile.js")))
     (should (string= lang1 "php"))
     (should (string= lang1b "php"))
@@ -122,16 +121,34 @@
          (expected (concat "ag --nocolor --nogroup --elisp --ignore-dir this/is/excluded " (shell-quote-argument expected-regexes) " /path/to/proj-root")))
     (should (string= expected  (dumb-jump-generate-ag-command  "tester" "blah.el" "/path/to/proj-root" regexes "elisp" '("/path/to/proj-root/this/is/excluded"))))))
 
+(ert-deftest dumb-jump-generate-git-grep-plus-ag-command-no-ctx-test ()
+  (let* ((regexes (dumb-jump-get-contextual-regexes "elisp" nil 'ag))
+         (expected-regexes "\\((defun|cl-defun)\\s+tester(?![a-zA-Z0-9\\?\\*-])|\\(defvar\\b\\s*tester(?![a-zA-Z0-9\\?\\*-])|\\(defcustom\\b\\s*tester(?![a-zA-Z0-9\\?\\*-])|\\(setq\\b\\s*tester(?![a-zA-Z0-9\\?\\*-])|\\(tester\\s+|\\((defun|cl-defun)\\s*.+\\(?\\s*tester(?![a-zA-Z0-9\\?\\*-])\\s*\\)?")
+         (expected (concat "ag --nocolor --nogroup -G '(/path/to/proj-root/blah.el)' " (shell-quote-argument expected-regexes) " /path/to/proj-root"))) ;; NOTE no "--elisp" and the `-G` arg is new
+  (with-mock
+   (mock (dumb-jump-get-git-grep-files-matching-symbol-as-ag-arg * *) => "'(/path/to/proj-root/blah.el)'")
+    (should (string= expected  (dumb-jump-generate-git-grep-plus-ag-command  "tester" "blah.el" "/path/to/proj-root" regexes "elisp" nil))))))
+
+
+(ert-deftest dumb-jump-generate-git-grep-plus-ag-command-exclude-test ()
+  (let* ((regexes (dumb-jump-get-contextual-regexes "elisp" nil 'ag))
+         (expected-regexes "\\((defun|cl-defun)\\s+tester(?![a-zA-Z0-9\\?\\*-])|\\(defvar\\b\\s*tester(?![a-zA-Z0-9\\?\\*-])|\\(defcustom\\b\\s*tester(?![a-zA-Z0-9\\?\\*-])|\\(setq\\b\\s*tester(?![a-zA-Z0-9\\?\\*-])|\\(tester\\s+|\\((defun|cl-defun)\\s*.+\\(?\\s*tester(?![a-zA-Z0-9\\?\\*-])\\s*\\)?")
+         (expected (concat "ag --nocolor --nogroup -G '(/path/to/proj-root/blah.el)' --ignore-dir this/is/excluded " (shell-quote-argument expected-regexes) " /path/to/proj-root"))) ;; NOTE no "--elisp" and the `-G` arg is new
+  (with-mock
+   (mock (dumb-jump-get-git-grep-files-matching-symbol-as-ag-arg * *) => "'(/path/to/proj-root/blah.el)'")
+    (should (string= expected  (dumb-jump-generate-git-grep-plus-ag-command  "tester" "blah.el" "/path/to/proj-root" regexes "elisp" '("/path/to/proj-root/this/is/excluded")))))))
+
+
 (ert-deftest dumb-jump-generate-rg-command-no-ctx-test ()
   (let* ((regexes (dumb-jump-get-contextual-regexes "elisp" nil 'rg))
          (expected-regexes "\\((defun|cl-defun)\\s+tester($|[^a-zA-Z0-9\\?\\*-])|\\(defvar\\b\\s*tester($|[^a-zA-Z0-9\\?\\*-])|\\(defcustom\\b\\s*tester($|[^a-zA-Z0-9\\?\\*-])|\\(setq\\b\\s*tester($|[^a-zA-Z0-9\\?\\*-])|\\(tester\\s+|\\((defun|cl-defun)\\s*.+\\(?\\s*tester($|[^a-zA-Z0-9\\?\\*-])\\s*\\)?")
-         (expected (concat "rg --color never --no-heading --line-number --type elisp " (shell-quote-argument expected-regexes) " .")))
+         (expected (concat "rg --color never --no-heading --line-number -U --pcre2 --type elisp " (shell-quote-argument expected-regexes) " .")))
     (should (string= expected  (dumb-jump-generate-rg-command  "tester" "blah.el" "." regexes "elisp" nil)))))
 
 (ert-deftest dumb-jump-generate-rg-command-remote-test ()
   (let* ((regexes (dumb-jump-get-contextual-regexes "elisp" nil 'rg))
          (expected-regexes "\\((defun|cl-defun)\\s+tester($|[^a-zA-Z0-9\\?\\*-])|\\(defvar\\b\\s*tester($|[^a-zA-Z0-9\\?\\*-])|\\(defcustom\\b\\s*tester($|[^a-zA-Z0-9\\?\\*-])|\\(setq\\b\\s*tester($|[^a-zA-Z0-9\\?\\*-])|\\(tester\\s+|\\((defun|cl-defun)\\s*.+\\(?\\s*tester($|[^a-zA-Z0-9\\?\\*-])\\s*\\)?")
-         (expected (concat "rg --color never --no-heading --line-number --type elisp -g \\!this/is/excluded " (shell-quote-argument expected-regexes) " /path/to/proj-root")))
+         (expected (concat "rg --color never --no-heading --line-number -U --pcre2 --type elisp -g \\!this/is/excluded " (shell-quote-argument expected-regexes) " /path/to/proj-root")))
     (should (string= expected  (dumb-jump-generate-rg-command  "tester" "blah.el" "/path/to/proj-root" regexes "elisp" '("/path/to/proj-root/this/is/excluded"))))))
 
 (ert-deftest dumb-jump-generate-git-grep-command-no-ctx-test ()
@@ -141,8 +158,33 @@
          (expected (concat "git grep --color=never --line-number --untracked -E " (shell-quote-argument expected-regexes) " -- ./\\*.el ./\\*.el.gz \\:\\(exclude\\)one \\:\\(exclude\\)two \\:\\(exclude\\)three")))
     (should (string= expected  (dumb-jump-generate-git-grep-command  "tester" "blah.el" "." regexes "elisp" excludes)))))
 
+(ert-deftest dumb-jump-generate-git-grep-command-no-ctx-extra-args ()
+  (let* ((regexes (dumb-jump-get-contextual-regexes "elisp" nil 'git-grep))
+         (expected-regexes "\\((defun|cl-defun)\\s+tester($|[^a-zA-Z0-9\\?\\*-])|\\(defvar\\b\\s*tester($|[^a-zA-Z0-9\\?\\*-])|\\(defcustom\\b\\s*tester($|[^a-zA-Z0-9\\?\\*-])|\\(setq\\b\\s*tester($|[^a-zA-Z0-9\\?\\*-])|\\(tester\\s+|\\((defun|cl-defun)\\s*.+\\(?\\s*tester($|[^a-zA-Z0-9\\?\\*-])\\s*\\)?")
+         (excludes '("one" "two" "three"))
+         (dumb-jump-git-grep-search-args "--recurse-submodules")
+         (expected (concat "git grep --color=never --line-number --untracked --recurse-submodules -E " (shell-quote-argument expected-regexes) " -- ./\\*.el ./\\*.el.gz \\:\\(exclude\\)one \\:\\(exclude\\)two \\:\\(exclude\\)three")))
+    (should (string= expected  (dumb-jump-generate-git-grep-command  "tester" "blah.el" "." regexes "elisp" excludes)))))
+
+(ert-deftest dumb-jump-generate-ag-command-no-ctx-extra-args ()
+  ;; ag args
+  (let* ((regexes (dumb-jump-get-contextual-regexes "elisp" nil 'ag))
+         (expected-regexes "\\((defun|cl-defun)\\s+tester(?![a-zA-Z0-9\\?\\*-])|\\(defvar\\b\\s*tester(?![a-zA-Z0-9\\?\\*-])|\\(defcustom\\b\\s*tester(?![a-zA-Z0-9\\?\\*-])|\\(setq\\b\\s*tester(?![a-zA-Z0-9\\?\\*-])|\\(tester\\s+|\\((defun|cl-defun)\\s*.+\\(?\\s*tester(?![a-zA-Z0-9\\?\\*-])\\s*\\)?")
+         (dumb-jump-ag-search-args "--follow")
+         (expected (concat "ag --nocolor --nogroup --follow --elisp " (shell-quote-argument expected-regexes) " .")))
+    (should (string= expected  (dumb-jump-generate-ag-command  "tester" "blah.el" "." regexes "elisp" nil)))))
+
+(ert-deftest dumb-jump-generate-rg-command-no-ctx-extra-args ()
+  ;; rg-args
+  (let* ((regexes (dumb-jump-get-contextual-regexes "elisp" nil 'rg))
+         (expected-regexes "\\((defun|cl-defun)\\s+tester($|[^a-zA-Z0-9\\?\\*-])|\\(defvar\\b\\s*tester($|[^a-zA-Z0-9\\?\\*-])|\\(defcustom\\b\\s*tester($|[^a-zA-Z0-9\\?\\*-])|\\(setq\\b\\s*tester($|[^a-zA-Z0-9\\?\\*-])|\\(tester\\s+|\\((defun|cl-defun)\\s*.+\\(?\\s*tester($|[^a-zA-Z0-9\\?\\*-])\\s*\\)?")
+         (dumb-jump-rg-search-args "--no-pcre2 --follow")
+         (expected (concat "rg --color never --no-heading --line-number -U --no-pcre2 --follow --type elisp " (shell-quote-argument expected-regexes) " .")))
+    (should (string= expected  (dumb-jump-generate-rg-command  "tester" "blah.el" "." regexes "elisp" nil)))))
+
 (ert-deftest dumb-jump-generate-git-grep-command-not-search-untracked-test ()
-  (let* ((dumb-jump-git-grep-search-untracked nil)
+  (let* ((dumb-jump-git-grep-search-args "")
+         (dumb-jump-git-grep-search-untracked nil)
          (regexes (dumb-jump-get-contextual-regexes "elisp" nil 'git-grep))
          (expected-regexes "\\((defun|cl-defun)\\s+tester($|[^a-zA-Z0-9\\?\\*-])|\\(defvar\\b\\s*tester($|[^a-zA-Z0-9\\?\\*-])|\\(defcustom\\b\\s*tester($|[^a-zA-Z0-9\\?\\*-])|\\(setq\\b\\s*tester($|[^a-zA-Z0-9\\?\\*-])|\\(tester\\s+|\\((defun|cl-defun)\\s*.+\\(?\\s*tester($|[^a-zA-Z0-9\\?\\*-])\\s*\\)?")
          (excludes '("one" "two" "three"))
@@ -354,7 +396,6 @@
   (let* ((bad-rule '(:type "variable" :supports ("ag" "grep" "rg" "git-grep") :language "elisp" :regex "\\\(defvarJJJ\\b\\s*" :tests ("(defvar test ")))
          (dumb-jump-find-rules (cons bad-rule dumb-jump-find-rules))
          (rule-failures (dumb-jump-test-grep-rules)))
-    ;(message "%s" (prin1-to-string rule-failures))
     (should (= (length rule-failures) 1))))
 
 (when (dumb-jump-ag-installed?)
@@ -362,7 +403,6 @@
     (let* ((bad-rule '(:type "variable" :supports ("ag" "grep" "rg" "git-grep") :language "elisp" :regex "\\\(defvarJJJ\\b\\s*" :tests ("(defvar test ")))
            (dumb-jump-find-rules (cons bad-rule dumb-jump-find-rules))
            (rule-failures (dumb-jump-test-ag-rules)))
-                                        ;(message "%s" (prin1-to-string rule-failures))
       (should (= (length rule-failures) 1)))))
 
 (when (dumb-jump-rg-installed?)
@@ -370,7 +410,6 @@
     (let* ((bad-rule '(:type "variable" :supports ("ag" "grep" "rg" "git-grep") :language "elisp" :regex "\\\(defvarJJJ\\b\\s*" :tests ("(defvar test ")))
 	   (dumb-jump-find-rules (cons bad-rule dumb-jump-find-rules))
 	   (rule-failures (dumb-jump-test-rg-rules)))
-					;(message "%s" (prin1-to-string rule-failures))
       (should (= (length rule-failures) 1)))))
 
 (when (dumb-jump-git-grep-installed?)
@@ -378,8 +417,7 @@
     (let* ((bad-rule '(:type "variable" :supports ("ag" "grep" "rg" "git-grep") :language "elisp" :regex "\\\(defvarJJJ\\b\\s*" :tests ("(defvar test ")))
 	   (dumb-jump-find-rules (cons bad-rule dumb-jump-find-rules))
 	   (rule-failures (dumb-jump-test-git-grep-rules)))
-					(message "%s" (prin1-to-string rule-failures))
-                                        (should (= (length rule-failures) 1)))))
+      (should (= (length rule-failures) 1)))))
 
 (when (dumb-jump-git-grep-installed?)
   (ert-deftest dumb-jump-test-git-grep-rules-not-test () ;; :not tests
@@ -419,13 +457,12 @@
          (results '((:path "/usr/blah/test.txt" :line 54 :context "function thing()")
                     (:path "/usr/blah/test2.txt" :line 52 :context "var thing = function()" :target "a"))))
     (with-mock
-     (mock (helm-build-sync-source * :candidates * :persistent-action *))
-     (mock (helm * * :buffer "*helm dumb jump choices*") => "/test2.txt:52: var thing = function()")
-     (mock (dumb-jump-result-follow '(:path "/usr/blah/test2.txt" :line 52 :context "var thing = function()" :target "a")))
+     (mock (helm-build-sync-source * :action * :candidates * :persistent-action *))
+     (mock (helm * * :buffer "*helm dumb jump choices*"))
      (dumb-jump-prompt-user-for-choice "/usr/blah" results))))
 
 (ert-deftest dumb-jump-prompt-user-for-choice-correct-helm-persistent-action-test ()
-  (dumb-jump-helm-persist-action "dumb-jump.el:1: (defn status")
+  (dumb-jump-helm-persist-action '(:path "dumb-jump.el" :line 1 :context " (defn status"))
   (should (get-buffer " *helm dumb jump persistent*")))
 
 (ert-deftest dumb-jump-prompt-user-for-choice-correct-ivy-test ()
@@ -722,10 +759,37 @@
      ;; confirm memoization of the previous result
      (should (eq (dumb-jump-ag-installed?) t)))))
 
-(ert-deftest dumb-jump-rg-installed?-test ()
+(ert-deftest dumb-jump-git-grep-plus-ag-installed?-test ()
+  (let ((dumb-jump--git-grep-plus-ag-installed? 'unset)
+        (dumb-jump--ag-installed? 'unset)
+        (dumb-jump--git-grep-installed? 'unset))
+    (with-mock
+     ; this isn't ideal but combining the ag and git grep responses but this shouldn't matter in practice with :times 2
+     (mock (shell-command-to-string *) => "ag version 0.33.0\nfatal: no pattern given\n" :times 2)
+     (should (eq (dumb-jump-git-grep-plus-ag-installed?) t))
+     ;; confirm memoization of the previous result
+     (should (eq (dumb-jump-git-grep-plus-ag-installed?) t)))))
+
+(ert-deftest dumb-jump-rg-installed?-test-no ()
   (let ((dumb-jump--rg-installed? 'unset))
     (with-mock
      (mock (shell-command-to-string *) => "ripgrep 0.3.1\n" :times 1)
+     (should (not (eq (dumb-jump-rg-installed?) t)))
+     ;; confirm memoization of the previous result
+     (should (not (eq (dumb-jump-rg-installed?) t))))))
+
+(ert-deftest dumb-jump-rg-installed?-test-yes ()
+  (let ((dumb-jump--rg-installed? 'unset))
+    (with-mock
+     (mock (shell-command-to-string *) => "ripgrep 0.10.0\n" :times 1)
+     (should (eq (dumb-jump-rg-installed?) t))
+     ;; confirm memoization of the previous result
+     (should (eq (dumb-jump-rg-installed?) t)))))
+
+(ert-deftest dumb-jump-rg-installed?-test-yes2 ()
+  (let ((dumb-jump--rg-installed? 'unset))
+    (with-mock
+     (mock (shell-command-to-string *) => "ripgrep 1.1.0\n" :times 1)
      (should (eq (dumb-jump-rg-installed?) t))
      ;; confirm memoization of the previous result
      (should (eq (dumb-jump-rg-installed?) t)))))
@@ -808,6 +872,11 @@
   (should (equal (dumb-jump-populate-regexes "testvar" '("JJJ\\s*=\\s*") 'ag) '("testvar\\s*=\\s*")))
   (should (equal (dumb-jump-populate-regexes "$testvar" '("JJJ\\s*=\\s*") 'ag) '("\\$testvar\\s*=\\s*"))))
 
+(ert-deftest dumb-jump-populate-regexes-git-grep-plus-ag-test ()
+  ;; this is effectively the same as `ag even with 'git-grep-plus-ag since that's where the regexes are used in this mode
+  (should (equal (dumb-jump-populate-regexes "testvar" '("JJJ\\s*=\\s*") 'git-grep-plus-ag) '("testvar\\s*=\\s*")))
+  (should (equal (dumb-jump-populate-regexes "$testvar" '("JJJ\\s*=\\s*") 'git-grep-plus-ag) '("\\$testvar\\s*=\\s*"))))
+
 (ert-deftest dumb-jump-populate-regexes-rg-test ()
   (should (equal (dumb-jump-populate-regexes "testvar" '("JJJ\\s*=\\s*") 'rg) '("testvar\\s*=\\s*")))
   (should (equal (dumb-jump-populate-regexes "$testvar" '("JJJ\\s*=\\s*") 'rg) '("\\$testvar\\s*=\\s*")))
@@ -838,10 +907,16 @@
 (ert-deftest dumb-jump-process-symbol-by-lang-test ()
   (let ((result (dumb-jump-process-symbol-by-lang "elisp" "somefunc"))
         (result2 (dumb-jump-process-symbol-by-lang "clojure" "myns/myfunc"))
-        (result3 (dumb-jump-process-symbol-by-lang "ruby" ":myrubyfunc")))
+        (result3 (dumb-jump-process-symbol-by-lang "ruby" ":myrubyfunc"))
+        (result3b (dumb-jump-process-symbol-by-lang "ruby" "Health::Checks::QueueGrowth"))
+        (result3c (dumb-jump-process-symbol-by-lang "ruby" "::Health"))
+        (result4 (dumb-jump-process-symbol-by-lang "systemverilog" "`myvlfunc")))
     (should (string= result "somefunc"))
     (should (string= result2 "myfunc"))
-    (should (string= result3 "myrubyfunc"))))
+    (should (string= result3 "myrubyfunc"))
+    (should (string= result3b "QueueGrowth"))
+    (should (string= result3c "Health"))
+    (should (string= result4 "myvlfunc"))))
 
 (ert-deftest dumb-jump--result-follow-test ()
   (let* ((data '(:path "/usr/blah/test2.txt" :line 52 :context "var thing = function()" :target "a")))
@@ -1067,6 +1142,11 @@
          (gen-funcs (dumb-jump-generators-by-searcher searcher)))
     (should (generators-valid gen-funcs searcher))))
 
+(ert-deftest dumb-jump-generators-by-searcher-git-grep-plus-ag ()
+  (let* ((searcher 'git-grep-plus-ag)
+         (gen-funcs (dumb-jump-generators-by-searcher searcher)))
+    (should (generators-valid gen-funcs searcher))))
+
 (ert-deftest dumb-jump-generators-by-searcher-rg ()
   (let* ((searcher 'rg)
          (gen-funcs (dumb-jump-generators-by-searcher searcher)))
@@ -1176,6 +1256,14 @@
      (mock (dumb-jump-prompt-user-for-choice "/code/redux" *))
      (dumb-jump-handle-results results "relfile.js" "/code/redux" "" "isNow" nil nil))))
 
+(ert-deftest dumb-jump-handle-results-non-aggressively-quick-look-test ()
+  (let ((dumb-jump-aggressive nil)
+        (results '((:path "relfile.js" :line 62 :context "var isNow = true" :diff 7 :target "isNow")
+                   (:path "src/absfile.js" :line 69 :context "isNow = false" :diff 0 :target "isNow"))))
+    (with-mock
+     (mock (popup-menu* '("relfile.js:62: var isNow = true" "src/absfile.js:69: isNow = false")) :times 1)
+     (dumb-jump-handle-results results "relfile.js" "/code/redux" "" "isNow" t nil))))
+
 ;; Make sure it jumps when there's only one possiblity in non-aggressive mode.
 (ert-deftest dumb-jump-handle-results-non-aggressive-do-jump-test ()
   (let ((dumb-jump-aggressive nil)
@@ -1252,3 +1340,22 @@
       (with-mock
        (mock (dumb-jump-goto-file-line * 5 7))
        (should (string= clj-to-file (dumb-jump-go)))))))
+
+
+(ert-deftest dumb-jump-format-files-as-ag-arg-test ()
+  (let* ((fake-files '("one" "two" "three"))
+         (result (dumb-jump-format-files-as-ag-arg fake-files "path"))
+         (expected "'(path/one|path/two|path/three)'"))
+    (should (string= result expected))))
+
+(ert-deftest dumb-jump-get-git-grep-files-matching-symbol-test ()
+  (with-mock
+   (mock (shell-command-to-string "git grep --full-name -F -c symbol path") => "fileA:1\nfileB:2\n")
+   (should (equal (dumb-jump-get-git-grep-files-matching-symbol "symbol" "path") '("fileA" "fileB")))))
+
+(ert-deftest dumb-jump-get-git-grep-files-matching-symbol-as-ag-arg-test ()
+  (with-mock
+   (mock (shell-command-to-string "git grep --full-name -F -c symbol path") => "fileA:1\nfileB:2\n")
+   (should (string= (dumb-jump-get-git-grep-files-matching-symbol-as-ag-arg "symbol" "path") "'(path/fileA|path/fileB)'"))))
+
+;;;
